function detect = FDI_detect(thresh, e, detvar, detcomp)

%FDI_DETECT is the function for the detection stage of the FDI filter
%   This file compares the error signal (e) agains the thresholds (thresh).
%   Input definition:
%   thresh  = thresholds
%   e       = residuals
%   detvar  = detection variables
%   detcomp = detection comparison vector

% Setting local flags for each time step of the error signal
% flag = +1 if +T <= error
% flag = 0 if -T < error < +T
% flag = -1 if error <= -T

% Setting global flags for the detection algorithm
% flag = +1 ==> positive rudder faults
% flag = 0 ==> no fault detected
% flag = -1 ==> negative rudder faults

%% Placeholder for flag
detect.global = [];
detect.frame.beta = [];
detect.frame.phi = [];
detect.frame.psi = [];
detect.frame.p = [];
detect.frame.r = [];
detect.frame.rud = [];
detect.frame.rail = [];
field = fieldnames(detect.frame);

%% Frame-by-frame hitting check
% This cell checks if each residual crosses its respective threshold
% frame-by-frame.
for k = 1:numel(e.time)
    for kk = 1:numel(field)
        if +thresh.(field{kk}) <= e.(field{kk})(k)
            detect.frame.(field{kk})(k) = +1;
        elseif -thresh.(field{kk}) < e.(field{kk})(k) && e.(field{kk})(k) < +thresh.(field{kk})
            detect.frame.(field{kk})(k) = 0;
        elseif e.(field{kk})(k) <= -thresh.(field{kk})
            detect.frame.(field{kk})(k) = -1;
        else
            fprintf(['\n There is an error in evaluating thresholds for ', (field{kk}), '.\n']);
        end
    end
end

%% Frame-by-frame hitting time check
% Identify the first instance where the residual crosses the threshold.
for kk = 1:numel(field)
    detect.hittime.frame.(field{kk}) = ...
        e.time(find(abs(detect.frame.(field{kk})) == 1, 1, 'first'));
end
detect.hittime.frame;

%% Sustained hitting check
% Check if the local flags stay at +/-1 for several consecutive frames. If
% the flags are +/-1 for a sustained period, a flag is declared.
nframe = 6; % number of frames to check for sustained hitting
for k = nframe:numel(e.time)
    for kk = 1:numel(field)
        if prod(detect.frame.(field{kk})(k-(nframe-1):k) == +1*ones(1,nframe))
            detect.sustain.(field{kk})(k) = 1;
        elseif prod(detect.frame.(field{kk})(k-(nframe-1):k) == -1*ones(1,nframe))
            detect.sustain.(field{kk})(k) = -1;
        else
            detect.sustain.(field{kk})(k) = 0;
        end
    end
end

%% Sustained hitting time check
% Identify the first instance when the local flags are +/-1 for a sustained
% period.
for kk = 1:numel(field)
    detect.hittime.sustain.(field{kk}) = ...
        e.time(find(abs(detect.sustain.(field{kk})) == 1, 1, 'first'));
end
detect.hittime.sustain;

%% Global hitting check
% Compare the local flags with the fault detection vector "detcomp" and set
% the global flag to +/-1 if a fault is detected.
for k = 1:numel(e.time)
    if prod([detect.sustain.(detvar{1})(k), detect.sustain.(detvar{2})(k), detect.sustain.(detvar{3})(k)] == detcomp)
        detect.global(k) = 1;
    elseif prod([detect.sustain.(detvar{1})(k), detect.sustain.(detvar{2})(k), detect.sustain.(detvar{3})(k)] == -1.*detcomp)
        detect.global(k) = -1;
    else
        detect.global(k) = 0;
    end
end

%% Global hitting time check
% Identify the first instance when the global flag becomes +/-1
detect.hittime.globalind = find(abs(detect.global) == 1, 1, 'first');
detect.hittime.global = e.time(detect.hittime.globalind);

end